//
//  EDAMClient.h
//  Evernote
//
//  Created by Andrew McGeachie on 9/18/07.
//  Copyright 2007 Evernote Corporation. All rights reserved.
//

#ifndef EDAMHTTPCLient_HEADER_GUARD
#define EDAMHTTPCLient_HEADER_GUARD


#import "EDAMClient.h"

@class EDAMNoteLockingClient;
@class ENAccountContext;
@class ENNoteMO;
@class THTTPClient;

extern NSString * const EDAMHTTPClientSyncChunkSessionIdentifier;
extern NSString * const EDAMHTTPClientNoteContentSessionIdentifier;
extern NSString * const EDAMHTTPClientNoteUploadSessionIdentifier;

@interface EDAMHTTPClient : NSObject <EDAMClient, NSCopying> {
  NSString * _username;
  NSString * _password;
  NSURL *  _userStoreShardURL;
  NSURL *  _userStoreBaseURL;
  NSURL *  _noteStoreBaseURL;
  NSURL *  _noteStoreShardURL;
  NSString * _authCacheKey;
  NSString * _shardID;
  NSString *_userAgent;
  
  id<TTransport> _userStoreTransport;
  id<TTransport> _noteStoreTransport;
  id<TTransport> _noteStoreTransportForBackground;
  id<TTransport> _noteStoreTransportForBackgroundForContent;
  id<TTransport> _noteStoreTransportForNoteUpload;
  id<TTransport> _catalogClientTransport;
  id<TTransport> _commerceClientTransport;
  id<TTransport> _utilityClientTransport;

  EDAMNoteLockingClient *_noteLockingClient;
  EDAMNoteStoreClient * _noteStore;
  EDAMNoteStoreClient * _noteStoreForBackgroundSync;
  EDAMNoteStoreClient * _noteStoreForBackgroundSyncForContent;
  EDAMNoteStoreClient * _noteStoreForNoteUpload;
  EDAMUserStoreClient * _userStore;
  EDAMCatalogClient * _catalogClient;
  EDAMCommerceClient * _commerceClient;
  EDAMUtilityClient * _utilityClient;
  
  BOOL _updatesNoteStoreURLWithAuthShard;
  
  dispatch_queue_t _noteStoreDispatchQueue;
  dispatch_queue_t _userStoreDispatchQueue;
  dispatch_queue_t _catalogClientDispatchQueue;
  dispatch_queue_t _commerceClientDispatchQueue;
  dispatch_queue_t _utilityClientDispatchQueue;
  
#if EDAMCLIENT_INCLUDE_MESSAGE_CLIENT
  NSURL * _messageStoreBaseURL;
  NSURL * _messageStoreShardURL;
  id<TTransport> _messageStoreTransport;
  EDAMMessageStoreClient * _messageStore;
  dispatch_queue_t _messageStoreDispatchQueue;
#endif
}

@property (strong, nonatomic) NSString * userAgent;
@property (strong, nonatomic) NSString * username;
@property (strong, nonatomic) NSString * password;
@property (strong, nonatomic) NSURL * userStoreBaseURL;
@property (strong, nonatomic) NSURL * noteStoreBaseURL;
@property (strong, nonatomic) NSURL * userStoreShardURL;
@property (strong, nonatomic) NSURL * noteStoreShardURL;
@property (readonly, nonatomic) THTTPClient * noteStoreHTTPTransport;
@property (readonly) THTTPClient * noteStoreHTTPTransportForBackgroundSync;

@property (readonly, nonatomic) EDAMNoteLockingClient *noteLockingClient;

@property (readonly, nonatomic) NSString * shardID;
@property (readonly, nonatomic) NSDate * authReceivedAt;
@property (readonly, nonatomic) EDAMAuthenticationResult * authResult;
@property (weak , nonatomic) ENAccountContext* mainAccountContext;

@property (readonly, nonatomic) id <EDAMNoteStore> noteStoreForBackgroundSync;
@property (readonly, nonatomic) id <EDAMNoteStore> noteStoreForBackgroundSyncForNoteContent;
@property (readonly, nonatomic) id <EDAMNoteStore> noteStoreForNoteUpload;

/**
 * By default, when we authenticate, we update the notestore and userstore
 * URLs to include the correct shard.  Setting this to NO turns this feature
 * off for the notestore URL.  This allows us to get sync state and chunks
 * for shared notebooks, using one shard for the userstore, and another shard
 * for the notestore.
 */
@property BOOL updatesNoteStoreURLWithAuthShard;

+ (void) setClientSecret:(NSString *)clientKey;
+ (void) setClientKey:(NSString *)clientKey;
+ (NSString *) clientVersionString;

+ (NSString *)deviceDescription;
+ (NSString *)deviceIdentifier;

- (id) initWithUsername: (NSString *) username
               password: (NSString *) password
           userStoreURL: (NSURL *) userStoreURL
           noteStoreURL: (NSURL *) noteStoreURL;

- (id) initWithUsername: (NSString *) username
               password: (NSString *) password
            serviceHost: (NSString *) serviceHost;


- (id) initWithLongLivedAuthResult:(EDAMAuthenticationResult *)authResult
                       serviceHost:(NSString *)serviceHost;

/**
 * Set the shard ID for the note store
 */
- (void) updateNoteStoreURLWithShard: (NSString *) shardID;


/**
 * Set the shard DI for the user store
 */
- (void) updateUserStoreURLWithShard: (NSString *) sharedID;


/**
 * Has this EDAMAuthenticationResult expired?
 */
- (BOOL) authResultHasExpired: (EDAMAuthenticationResult *) result
                   receivedAt: (NSDate *) receivedAt;


/**
 * Determine if this EDAMAuthenticationResult is close enough to expiring 
 * that we should refresh it...
 */
- (BOOL) authResultNeedsRefresh: (EDAMAuthenticationResult *) result
                     receivedAt: (NSDate *) receivedAt;


/**
 * Removes all authentication results from the cache
 */
- (void) clearAuthCache;

/**
 * Returns an NSMutableURLRequest for the given NSURL, 
 * with the authToken set as the cookie.  This is used
 * when talking to actions on the noteStore (e.g. /clip.action)
 */
- (NSMutableURLRequest *) authenticatedURLRequestForURL:(NSURL *)url;

- (NSURL *) urlForDisplayingSharedNote:(ENNoteMO *)note;
- (NSURL *) shortURLForDisplayingSharedNote:(ENNoteMO *)note encodedAdditionalString:(NSString *)encodedAdditionalString;

- (void) revokeLongLivedAuthToken;

- (EDAMAuthenticationResult *) reauthenticate;

- (EDAMAuthenticationResult *) completeTwoFactorAuth:(NSString *)authToken securityKey:(NSString *)securityKey;

/**
 * Cancels all note content tasks scheduled by NSURLSession
 */
- (void) cancelNoteContentTasks;

/**
 * Cancels all note sync tasks scheduled by NSURLSession
 */
- (void) cancelSyncChunkTasks;


/**
 * User agent convenience method
 */
+ (NSString *) userAgent;


/**
 * Clear the auth token for this client
 */
- (void) clearAuthToken;


@end

#endif
