//
//  ENAccountContext.h
//  CoreNote
//
//  Created by Steve White on 5/15/12.
//  Copyright (c) 2012 Evernote Corporation. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "ENCoreDataLocalStore.h"
#import "ENCoreDataLocalStoreBackingDelegate.h"
#import "ENSyncEngine.h"

@class EDAMClientCache;
@class EDAMSharedNotebookHTTPClient;
@class ENCacheManager;
@class ENPreviewManager;
@class ENReverseGeocodingManager;
@class ENSegmentedDataCache;
@class ENSyncEngine;
@class ENSyncState;
@class EDAMHTTPClient;
@class NSManagedObjectID;


typedef enum {
  ENAccountContextSaveType_None,
  ENAccountContextSaveType_Now,
  ENAccountContextSaveType_Idle
} ENAccountContextSaveType;


typedef void (^ENAccountContextLocalStoreReadOnlyWorkerBlock)(ENCoreDataLocalStore *localStore);
typedef void (^ENAccountContextLocalStoreMOIDReadOnlyWorkerBlock)(ENCoreDataLocalStore *localStore, id managedObject);
typedef ENAccountContextSaveType (^ENAccountContextLocalStoreWorkerBlock)(ENCoreDataLocalStore *localStore);
typedef ENAccountContextSaveType (^ENAccountContextLocalStoreMOIDWorkerBlock)(ENCoreDataLocalStore *localStore, id managedObject);
typedef void (^ENAccountContextLocalStoreCompletionBlock)(BOOL success, NSError *error);
typedef void (^ENAccountContextEmailVerificationCompletionBlock)(BOOL success);

// Notification posted by ENAccountContext if service reachability changes
// (Only sent if -sendsServiceReachabilityNotifications return YES)
extern NSString *const ENAccountContextServiceReachabilityDidChangeNotification;

@interface ENAccountContext : NSObject <ENCoreDataLocalStoreBackingDelegate> {
  BOOL _anonymousContext;
  
  ENCoreDataLocalStore *_readOnlyLocalStore;
  ENCoreDataLocalStore * _readWriteLocalStore;
  
  NSArray *_directoryPathComponents;
  NSString *_cacheDirectory;
  NSString *_documentsDirectory;
  NSString *_contentDirectory;
  BOOL _unifiedFileSystem;
  
  ENSyncEngine *_syncEngine;
  ENSyncState *_syncState;
  int32_t _automaticSyncDisabled;
  
  EDAMClientCache *_edamClientCache;
  EDAMHTTPClient *_edamClient;
  ENCacheManager *_cacheManager;
  
  ENReverseGeocodingManager *_geocodingManager;
  
  ENPreviewManager *_previewManager;
  ENSegmentedDataCache *_dataCache;
}

@property (readonly, nonatomic) EDAMUser *userInfo;

@property (strong, nonatomic, readonly) NSNumber *businessID;

@property (readonly, nonatomic) ENCoreDataLocalStore *readOnlyLocalStore;
@property (readonly, nonatomic) ENCoreDataLocalStore *readWriteLocalStore;
@property (readonly, nonatomic) id<ENSearchStore> searchStore;
@property (readonly, nonatomic) id<ENMetadataStore> metadataStore;

@property (readonly, nonatomic) ENSyncEngine *syncEngine;
@property (weak, nonatomic) id<ENSyncEngineConflictDelegate> syncEngineConflictDelegate;
@property (readonly, nonatomic) ENSyncState *syncState;
@property (readonly, nonatomic) EDAMHTTPClient *edamClient;
@property (readonly, nonatomic) EDAMHTTPClient *edamNoteStoreClientForBusiness;
@property (readonly, nonatomic) EDAMClientCache *edamClientCache;
@property (assign) NSTimeInterval autoSyncInterval;


- (EDAMHTTPClient *) edamNoteStoreClientForNotebookGUID:(NSString *)notebookGUID;

@property (readonly, nonatomic) ENCacheManager *cacheManager;
- (BOOL) shouldUseSessionDownloader;

@property (readonly, nonatomic) NSString* serviceHost;

// Indicates that the service is reachable
// Do not observe this property. Register for the ENAccountContextServiceReachabilityDidChangeNotification instead.
@property (readonly) BOOL serviceReachable;

// If set to true, the account posts a notification when reachability changes. Default is NO.
@property (nonatomic) BOOL sendsServiceReachabilityNotifications;

@property (readonly, nonatomic) NSString *activityLogFile;

@property (readonly, nonatomic) BOOL hasConfiguredGeocodingManager;
@property (readonly, nonatomic) ENReverseGeocodingManager *geocodingManager;

@property (readonly, nonatomic) ENPreviewManager *previewManager;
@property (readonly, nonatomic) ENSegmentedDataCache *dataCache;

@property (nonatomic, readonly, getter=isPremiumAccount) BOOL premiumAccount;

// One could think of this as an "is activated business account"
// It returns YES only if the user is part of a business, and if
// SSO is required that they've completed the SSO requirements.
@property (nonatomic, readonly, getter=isBusinessAccount) BOOL businessAccount;

// isBusinessAccount will return NO if the user is part of a business,
// but hasn't yet done the necessary SSO.  This method will return
// YES in that scenario, and NO in all others.
- (BOOL) needsBusinessSSO;

// Returns YES if SSO authentication has completed once.
- (BOOL) businessSSOHasCompletedOnce;

+ (NSString *) baseContentPath;

+ (void) setBaseDocumentPath:(NSString *)documentsPath;
+ (void) setBaseCachePath:(NSString *)cachePath;

- (void) disableAutomaticSync;
- (void) enableAutomaticSync;
- (BOOL) isAutomaticSyncEnabled;

#if defined ANONYMOUS_CONTEXT_SUPPORT && ANONYMOUS_CONTEXT_SUPPORT != 0
- (id) initForAnonymousUser;
- (BOOL) isAnonymousContext;
- (void) convertAnonymousWithLongLivedAuthResult:(EDAMAuthenticationResult *)authResult
                                   onServiceHost:(NSString *)serviceHost;
#endif

#if defined APPLE_INSTORE_DEMO && APPLE_INSTORE_DEMO != 0
- (id) initForAppleStoreDemo;
#endif

- (id) initWithLongLivedAuthResult:(EDAMAuthenticationResult *)authResult
                       serviceHost:(NSString *)serviceHost
                 unifiedFileSystem:(BOOL)unifiedFileSystem;


- (id) initWithUsername:(NSString *)username
               password:(NSString *)password
						serviceHost:(NSString *)serviceHost;

- (id) initWithUsername:(NSString *)username
               password:(NSString *)password
						serviceHost:(NSString *)serviceHost
      unifiedFileSystem:(BOOL)unifiedFileSystem;

- (void) closeContext;
- (void) logout;
- (void) sync;
- (void) backgroundSessionDidResume;

- (NSString *) cacheDirectory;
- (NSString *) documentsDirectory;
- (NSString *) contentDirectory;

- (void)verifyEmailAddress:(ENAccountContextEmailVerificationCompletionBlock)completion;
- (void)confirmEmailAddress:(ENAccountContextEmailVerificationCompletionBlock)completion;

//
// Local store helper methods
//

// Asynchronous read only blocks, available to call from any thread.
// Attempts to invoke localStore save: will result in an assertion.
- (void)performLocalStoreReadBlock:(ENAccountContextLocalStoreReadOnlyWorkerBlock)block;
- (void)performWithObjectID:(NSManagedObjectID *)moid
        localStoreReadBlock:(ENAccountContextLocalStoreMOIDReadOnlyWorkerBlock)block;

// Synchronous read only blocks, available to call from any thread.
// Attempts to invoke localStore save: will result in an assertion.
- (void)performLocalStoreReadBlockAndWait:(ENAccountContextLocalStoreReadOnlyWorkerBlock)block;
- (void)performWithObjectID:(NSManagedObjectID *)moid
 localStoreReadBlockAndWait:(ENAccountContextLocalStoreMOIDReadOnlyWorkerBlock)block;

// Asynchronous read write blocks, available to call from any thread.
- (void)performLocalStoreReadWriteBlock:(ENAccountContextLocalStoreWorkerBlock)block
                             completion:(ENAccountContextLocalStoreCompletionBlock)completionBlock;
- (void)performWithObjectID:(NSManagedObjectID *)moid
   localStoreReadWriteBlock:(ENAccountContextLocalStoreMOIDWorkerBlock)block
                 completion:(ENAccountContextLocalStoreCompletionBlock)completionBlock;

// Synchronous read write blocks, *cannot* be called from the main thread.
- (void)performLocalStoreReadWriteBlockAndWait:(ENAccountContextLocalStoreWorkerBlock)block
                                    completion:(ENAccountContextLocalStoreCompletionBlock)completionBlock;
- (void)performWithObjectID:(NSManagedObjectID *)moid
localStoreReadWriteBlockAndWait:(ENAccountContextLocalStoreMOIDWorkerBlock)block
                 completion:(ENAccountContextLocalStoreCompletionBlock)completionBlock;


/**
 * Unique ID for this context
 */
@property (readonly) NSString * contextID;


/**
 * Construct our read-write localstore.  Handy for overriding by subclasses.
 * In fact, really only to be used by subclasses.
 */
- (ENCoreDataLocalStore *) instantiateReadWriteLocalStore;


#if !TARGET_OS_IPHONE

/**
 * Determine how many accounts are stored locally for the given profile
 */
+ (NSUInteger) countOfLocalAccountsForProfile: (EDAMBootstrapProfile *) profile;


#endif //!TARGET_OS_IPHONE


/**
 *
 * ===================================================================================================
 * YOU SHOULDN'T NEED TO USE THE FOLLOWING METHODS UNLESS YOU ARE SUBCLASSING OR DOING SOMETHING ELITE
 * ===================================================================================================
 *
 */
- (ENSyncState *) reconstituteSyncState;


@end
