//
//  ENSyncEngine+Private.h
//  Evernote
//
//  Created by Steve White on 4/28/09.
//  Copyright 2009 Evernote Corporation. All rights reserved.
//

#ifndef ENSyncEngineCategories_HEADER_GUARD
#define ENSyncEngineCategories_HEADER_GUARD

#import <Foundation/Foundation.h>
#import "ENSyncEngine.h"

typedef enum {
  ENSyncEngineErrorHandlingStop,
  ENSyncEngineErrorHandlingRetry,
  ENSyncEngineErrorHandlingContinue,
} ENSyncEngineErrorHandling;

@interface ENSyncEngine ()

/**
 * an atomic property for internal use.  safe to use from any thread.  this allows "isExecuting" to be used for bindings
 * or UI KVO updating (this avoids creating a race condition when jumping between threads).
 */
@property BOOL internalIsExecuting;

@property (nonatomic) BOOL syncShouldCancel;
@property (nonatomic) BOOL executing;
@property BOOL needAdditionalIncrementalSync;
@property BOOL needAdditionalGlobalSync;
@property BOOL needsAuthRefresh;

@property int32_t currentSyncUpdateCount;
@property (strong, nonatomic) EDAMHTTPClient *edamNoteStoreClient;
@property (strong, nonatomic) ENSyncState *currentSyncState;
@property (strong, readwrite, nonatomic) NSString *statusMessage;
@property (assign, readwrite, nonatomic) float downloadProgress;
@property (assign, readwrite, nonatomic) float uploadProgress;
@property (nonatomic, readonly, getter = isPremiumAccount) BOOL premiumAccount;
@property (copy, nonatomic) void (^syncFinishedCallback)(ENSyncEngine *);
@property (strong, nonatomic) id uploadingEntityID;

- (EDAMHTTPClient *) edamNoteStoreClientForNotebookWithGUID:(NSString *)notebookGUID;
- (ENSyncState *) businessSyncState;

- (BOOL) authenticateWithServer;
- (NSString *) authTokenForSyncStateOrChunk;
- (NSString *) authToken;

- (BOOL) canAccessNetwork;
- (BOOL) performEDAMVersionCheck;

- (void) startSyncInit;
- (void) _clearState;
- (BOOL) setupStateForAccountID:(id)accountID;

- (void) _syncStarted;
- (void) _syncFinished;
- (void) _syncStoppedWithErrors:(NSArray *)errors;
- (BOOL) _downSyncServerChanges;
- (void) flagLinkedNotebookAsInvalid;

- (BOOL) _saveLocalStore:(NSError *__autoreleasing *)outError;
- (void) _createExpungeRecordForEntity:(NSString *)entity
                              withGuid:(NSString *)guid;
- (void) _addSyncError:(NSError *)error;

- (void) setLastSyncTime:(NSDate *)syncTime updateCount:(int32_t)updateCount;
- (void) _receivedNewUSNFromServer:(int32_t)newUSN;

- (BOOL) processSyncChunk:(EDAMSyncChunk *)chunk;

- (BOOL) postProcessChunk:(EDAMSyncChunk*)syncChunk
   withIncrementalSyncUSN:(int32_t)incrementalSyncUSN;

- (void) sanitizeAttributeStringsInEdamObject:(id)edamObject;

- (void) logNetworkStatus;
- (BOOL) checkIfWeCanSyncWithIgnoreNetworkPreference:(BOOL) ignoreNetworkPreference;
- (BOOL) isFullSyncRequiredWithServerWithServerSyncState:(EDAMSyncState*)serverSyncState;

- (BOOL) enumerateLocalExpungableGuids:(NSArray *)expungables
                            usingBlock:(int32_t(^)(NSString *guid))block
                            completion:(void(^)(NSArray *guidsExpunged))completion;

- (BOOL) enumerateLocalUploadableObjectIDs:(NSArray *)uploadables
                                usingBlock:(int32_t(^)(id objectID))block;

@end

@interface ENSyncEngine (ChunkDownloader)
- (uint16_t) preferredChunkSize;
- (void) clearCacheDownloadDirectory;
- (void) usingSyncChunkFilter:(EDAMSyncChunkFilter *)filter
 enumerateAllServerSyncChunks:(void(^)(EDAMSyncChunk *syncChunk, BOOL *stop))enumerationBlock;

- (EDAMSyncChunk *) getSyncChunkAfterUSN:(int32_t)currentUSN
                            fullSyncOnly:(BOOL)fullSyncOnly
                               chunkSize:(uint16_t)chunkSize
                            isBackground:(BOOL)isBackground;
- (BOOL) downloadAndProcessSyncChunksFullSyncAtUSN:(int32_t)fullSyncUSN
                              incrementalSyncAtUSN:(int32_t)incrementalSyncUSN;
@end

@interface ENSyncEngine (ChunkDownloaderSession)
- (BOOL) downloadAndProcessSyncChunksInSession;
- (BOOL) processSyncChunkFromBackgroundTransfer:(EDAMSyncChunk*)syncChunk;
@end

@interface ENSyncEngine (Errors)
- (void) _handleEDAMCallException:(NSException *)exception;

- (void) addSyncErrorForObjectWithID:(NSManagedObjectID *)objectID
                               title:(NSString *)title
                             message:(NSString *)message
                     underlyingError:(id)underlyingError;

- (void) removeSyncErrorForObjectID:(NSManagedObjectID *)objectID;
- (void) removeSyncErrorForObject:(NSManagedObject *)object;

- (void) addLocalStoreSaveError:(NSError *)saveError 
                forItemsInChunk:(NSArray *)chunkItems;
- (void) addLocalStoreSaveError:(NSError *)saveError 
      duringDeletionOfItemClass:(Class)itemClass;

- (ENSyncEngineErrorHandling) _errorHandlingTypeForException:(NSException *)exception;

@end

@interface ENSyncEngine (Notes)
- (void) initializeNotes;
+ (NSString *)lastUploadHashKeyForNote:(ENNoteMOID *)noteMoid;
- (BOOL) processLocalNoteChanges;
- (BOOL) syncNotes:(NSArray *)serverNotes;
- (BOOL) syncNotesFromChunk: (EDAMSyncChunk *) chunk;
- (void) updateLocalNote:(ENNoteMO *)localNote
          fromServerNote:(EDAMNote *)serverNote;

@end

@interface ENSyncEngine (LinkedNotebooks)

enum {
  kENSyncEngineLinkedNotebooksVersion0 = 0x00000000,
  kENSyncEngineLinkedNotebooksVersion1 = 0x00010000,
  kENSyncEngineLinkedNotebooksVersion2 = 0x00020000,
  kENSyncEngineLinkedNotebooksVersion  = kENSyncEngineLinkedNotebooksVersion2
};

- (BOOL) processLocalLinkedNotebookChanges;
- (BOOL) syncLinkedNotebooksFromChunk: (EDAMSyncChunk *) chunk;
- (BOOL) syncLinkedNotebooks:(NSArray *)serverNotebooks
                    forcibly:(BOOL)forcibly;
- (BOOL) syncAllLinkedNotebooks;
- (NSArray *) allServerLinkedNotebooks;
@end

@interface ENSyncEngine (Notebooks)

enum {
  kENSyncEngineNotebooksVersion0 = 0x00000000,
  kENSyncEngineNotebooksVersion1 = 0x00010000,
  kENSyncEngineNotebooksVersion2 = 0x00020000,
  kENSyncEngineNotebooksVersion3 = 0x00030000,
  kENSyncEngineNotebooksVersion4 = 0x00040000,
  kENSyncEngineNotebooksVersion  = kENSyncEngineNotebooksVersion4
};

- (NSArray *) allServerNotebooks;
- (void) updateLocalNotebook: (ENNotebookMO *) localNotebook
          fromServerNotebook: (EDAMNotebook *) serverNotebook;
- (EDAMNotebook *) serverNotebookWithGUID:(NSString *)guid;
- (int32_t) updateServerNotebookFromLocalNotebookWithID: (ENNotebookMOID *) localNotebookID;
- (BOOL) processLocalNotebookChanges;
- (BOOL) syncNotebooksFromChunk: (EDAMSyncChunk *) chunk;
- (BOOL) expungeNotebooksFromChunk:(EDAMSyncChunk *)chunk;
- (BOOL) syncNotebooks:(NSArray *)serverNotebooks
              forcibly:(BOOL)forcibly;
- (BOOL) syncAllNotebooks;
@end

@interface ENSyncEngine (Preferences)

enum {
  kENSyncEnginePreferencesVersion0 = 0x00000000,
  kENSyncEnginePreferencesVersion  = kENSyncEnginePreferencesVersion0
};

- (EDAMPreferences *) serverPreferencesWithNames:(NSArray *)names;
- (BOOL) syncPreferencesFromChunk:(EDAMSyncChunk *)chunk;
- (BOOL) processLocalPreferenceChanges;

@end

@interface ENSyncEngine (Resources)
- (EDAMLazyMap *) serverApplicationDataForResourceWithGUID:(NSString *)guid;
- (void) initializeResources;
- (BOOL) syncResources:(NSArray *)serverResources;
- (BOOL) syncResourcesFromChunk: (EDAMSyncChunk *) chunk;
- (void) updateLocalResource: (ENResourceMO *) localRsrc
          fromServerResource: (EDAMResource *) serverRsrc;
- (BOOL) updateServerResource:(EDAMResource *) serverRsrc
            fromLocalResource:(ENResourceMO *)localRsrc
    usingAlternateBackingFile:(NSString *)alternateBackingFile;
- (BOOL) processLocalResourceChanges;
@end

@interface ENSyncEngine (Searches)

enum {
  kENSyncEngineSavedSearchesVersion0 = 0x00000000,
  kENSyncEngineSavedSearchesVersion1 = 0x00001000,
  kENSyncEngineSavedSearchesVersion  = kENSyncEngineSavedSearchesVersion1
};

- (BOOL) processLocalSearchChanges;
- (BOOL) syncSearches:(NSArray *)serverSearches
             forcibly:(BOOL)forcibly;
- (BOOL) syncSearchesFromChunk:(EDAMSyncChunk *)chunk;
- (BOOL) syncAllSavedSearches;
@end

@interface ENSyncEngine (SharedNotebooks)

enum {
  kENSyncEngineSharedNotebooksVersion0 = 0x00000000,
  kENSyncEngineSharedNotebooksVersion1 = 0x00001000,
  kENSyncEngineSharedNotebooksVersion2 = 0x00002000,
  kENSyncEngineSharedNotebooksVersion  = kENSyncEngineSharedNotebooksVersion2
};

- (NSArray *) allServerSharedNotebooks;
- (void) updateServerSharedNotebook:(EDAMSharedNotebook *)serverShare
            fromLocalSharedNotebook:(ENSharedNotebookMO *)localShare;
- (void) updateLocalSharedNotebook:(ENSharedNotebookMO *)localShare
          fromServerSharedNotebook:(EDAMSharedNotebook *)serverShare;
- (BOOL) sendLocalSharedNotebookChanges:(NSArray *)shareObjectIDs;
- (BOOL) processLocalSharedNotebookChanges;
- (BOOL) syncAllSharedNotebooks;
@end

@interface ENSyncEngine (Tags)

enum {
  kENSyncEngineTagsVersion0 = 0x00000000,
  kENSyncEngineTagsVersion  = kENSyncEngineTagsVersion0
};

- (NSArray *) allServerTags;
- (NSArray *) allVisibleServerTags;
- (EDAMTag *) serverTagWithGUID:(NSString *)guid;
- (int32_t) updateServerTagFromLocalTagWithID: (ENTagMOID *) localTagID;
- (BOOL) processLocalTagChanges;
- (BOOL) syncTags:(NSArray *)serverTags
         forcibly:(BOOL)forcibly;
- (BOOL) syncTagsFromChunk:(EDAMSyncChunk *)chunk;
@end


@interface ENSyncEngine (Users)

enum {
  kENSyncEngineUserProfilesVersion1 = 0x00001000,
  kENSyncEngineUserProfilesVersion  = kENSyncEngineUserProfilesVersion1
};

- (BOOL) syncAllUserProfiles;

@end

#if TARGET_OS_IPHONE
@interface ENSyncEngine (Background)
- (void) performBackgroundSyncForAccountID:(id)accountID
                         pushedUpdateCount:(int32_t)updateCount
                         completionHandler:(void (^)(UIBackgroundFetchResult))completionHandler;
@end
#endif

#endif
