//
//  ENFileUtils+ENSharedUtilities.h
//  EverNote
//
//  Created by Andrew McGeachie on 11/25/08.
//  Copyright 2008 Evernote, Corp. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <CoreNote/CoreNote.h>


@interface ENFileUtils (ENSharedUtilities)


/**
 * Grab the mod time for a file
 */
+ (NSDate *) modTimeForFile: (NSString *) file;


/**
 * Receive files promised in a drag operation
 */
+ (NSArray *) receivePromisedFiles: (id<NSDraggingInfo>) dragInfo;


/**
 * Make sure the specified filename is unique by appending numbers to it.
 * @return The new filename (or the original, if unchanged)
 */
+ (NSString *) ensureUniqueFilename: (NSString *) filename;


/**
 * Set the file mod time to the current date and time
 */
+ (void) setModificationDateToNowForFile: (NSString *) file;


/**
 * Return a filename suitable for use as a temporary file.
 * The file does not exist.
 */
+ (NSString *) temporaryFile;

/**
 * Return a filename suitable for use as a temporary file,
 * with the given prefix and extension.
 * The file does not exist.
 * The filename will be in the format prefix*.ext
 */
+ (NSString *)temporaryFileWithPrefix:(NSString *)prefix extension:(NSString *)extension;


/**
 * Returns (and creates if it doesn't exist) the root directory where account backups are
 * stored.
 */
+ (NSString *)rootAccountBackupDir;


/**
 * Checks if a directory exists at the given path.
 * If a file exists at that path instead of a directory and moveIfNotDirectory is set, attempt to move the file.
 * If the move fails, outMoveFailed is set to YES.
 */
+ (BOOL)directoryExistsAtPath:(NSString *)path moveIfNotDirectory:(BOOL)moveIfNotDirectory moveFailed:(BOOL *)outMoveFailed;


/**
 * Create a directory at the given path, including intermediate directories.
 * The given path is tilde-expanded.
 * If a non-directory object exists at the given path, it is moved to a new name 
 * (via ensureUniqueFilename:).
 * If an error occurs, it is logged (via AppLogError).
 * @return YES on success, NO on failure.
 */
+ (BOOL)createDataDirectory:(NSString *)path;


/**
 * Return the contents of the given directory, in descending order of most-recent-modification
 * time.
 * If there is no directory at the given path, an empty array is returned.
 * @return Array of NSString: file names relative to path (as from NSFileManager's 
 * contentsOfDirectoryAtPath).
 */
+ (NSArray *)contentsOfDirectoryMostRecentFirst:(NSString *)path;


/** 
 * Log the number of bytes available on the volume containing the given path.
 * The message logged will look something like:
 * 2013-02-04 12:26:20 main [ENFileUtils] INFO: +[ENFileUtils logFreeDiskSpaceForVolumeContainingPath:messagePrefix:] -- /Users/beegeevee/Library/Application Support/Evernote/accounts/Evernote/beegeevee -- free space = 64625385472
 */
+ (void)logFreeDiskSpaceForVolumeContainingPath:(NSString *)path;


/** 
 * Log the number of bytes available on the volume containing the given path.
 * @param prefix If not nil, prefix is used instead of the value of __PRETTY_FUNCTION__ to prefix
 * the log message, e.g. (for a prefix parameger of @"FOOBAR"):
 * 2013-02-04 12:27:28 main [ENFileUtils] INFO: FOOBAR -- /Users/beegeevee/Library/Application Support/Evernote/accounts/Evernote/beegeevee -- free space = 64610594816
 */
+ (void)logFreeDiskSpaceForVolumeContainingPath:(NSString *)path messagePrefix:(NSString *)prefix;


/**
 * Get the number of bytes available on the volume containing the given path.
 * Returns 0 and logs an error message on failure.
 */
+ (unsigned long long)freeDiskSpaceForVolumeContainingPath:(NSString *)path;


/**
 * Directory where we put incoming files that are too large (~/Documents/Evernote Files).
 */
+ (NSString *)evernoteOverflowFilesDirectory;


/**
 * The icon for the system default web browser
 */
+ (NSImage*)defaultWebBrowserIconForMenus;

/**
 * The icon for the system default mail application
 */
+ (NSImage*)defaultMailApplicationIconForMenus;


/**
 *  Give a .webloc file, figure out its URL
 */
+ (NSString *)urlForWeblocFile:(NSString *)weblocFile;


/**
 * Determine the content type (as a Universal Type Identifier) for the specified file.
 */
+ (NSString *) determineMarkupTypeForFilename: (NSString *) filename;


/**
 * Return the process IDs for every process the has the specified file open
 */
+ (NSArray *) processIDsWithThisFileOpen: (NSString *) filepath
                          includeThisPID: (BOOL) includeSelf;


/**
 * Move the given file to the trash.
 * If that fails, delete the file.
 */
+ (void) trashFileAtPath: (NSString *) path;


/**
 * YES if the parameter is a file URL and the file is in the trash, otherwise NO.
 */
+ (BOOL)isFileURLInTrash:(NSURL *)url;


/**
 * Given a spotlight metadata file URL, figure out where the actual note
 * content is stored on disk.
 */
+ (NSString *) noteContentDirFromSpotlightMetaDataURL: (NSURL *) url;


+ (NSUInteger) unsignedIntFromFile: (NSString *) file;
+ (void) saveUnsignedInt: (NSUInteger) value toFile: (NSString *) file;


/**
 * Directory for storing logs
 */
+ (NSString *) logDirectory;


@end
